<?php
/**
 * This file implements the Mini Stats plugin.
 *
 */
if( !defined('EVO_MAIN_INIT') ) die( 'Please, do not access this page directly.' );


/**
 * Mini Stats Plugin
 *
 * This plugin displays a short list of stats about a blog
 *
 * @package plugins
 */
class ministats_plugin extends Plugin
{
	/**
	 * Variables below MUST be overriden by plugin implementations,
	 * either in the subclass declaration or in the subclass constructor.
	 */
	var $name = 'Mini Stats';
	var $code = 'ministats';
	var $priority = 80;
	var $version = '2.2.3-dev';
	var $author = 'Danny Ferguson';
	var $help_url = 'http://b2evo.astonishme.co.uk/index.php?title=mini_stats';
	var $group = 'AstonishMe';

	/*
	 * These variables MAY be overriden.
	 */
	var $is_tool = false;
	var $apply_when = 'never';

	function PluginInit( & $params)
	{
		$this->short_desc = T_('Brief stats for a blog');
		$this->long_desc = T_('Display a short list of stats about your blog.');
	}

	/**
	 * We require b2evo 2.2
	 */
	function GetDependencies()
	{
		return array(
				'requires' => array(
					'api_min' => array( 99 ), // obsolete, but required for b2evo 1.8 before 1.8.3
					'app_min' => '2.2.0-beta',
				),
			);
	}


	function get_widget_param_definitions( $params )
	{
		return array(
			'title' => array(
				'label' => $this->T_('Title'),
				//'note' => $this->T_('For which city would you like to display weather information?'),
				'size' => 24,
				'type' => 'text',
				'defaultvalue' => T_('Stats')
			),
			'summary' => array(
				'label' => $this->T_('Summary'),
				'type' => 'checkbox',
				'id' => 'ministats_summary',
				'defaultvalue' => 1
			),
			'summary_text' => array(
				'label' => $this->T_('Summary text'),
				'note' => $this->T_('The following tags will be replaced with the real stat value:  $post_count$,
				$comment_count$,
				$first_post_date$,
				$last_post_date$,
				$total_words$,
				$total_views$
				'),
				'type' => 'textarea',
				'rows' => 8,
				'defaultvalue' => 'This blog has $post_count$ posts and $comment_count$ comments spanning a range from $first_post_date$ to $last_post_date$.  The total number of words in all posts is $total_words$ and the total number of views for individual posts is $total_views$.',
				'id' => 'ministats_summary_text'
			),
			'most_comments' => array(
				'label' => $this->T_('Most comments'),
				'type' => 'checkbox',
				'defaultvalue' => 1,
				'id' => 'most_comments'
			),
			'comments_title' => array(
				'label' => $this->T_('Most comments title'),
				'type' => 'text',
				'defaultvalue' => 'Most comments',
				'size' => 30,
				'id' => 'ministats_comments_title'
			),
			'comments_limit' => array(
				'label' => $this->T_('Most comments limit'),
				'type' => 'integer',
				'defaultvalue' => 5,
				'size' => 3,
				'id' => 'ministats_comments_limit',
				'note' => $this->T_('How many posts should be in the list?')
			),
			'most_views' => array(
				'label' => $this->T_('Most views'),
				'type' => 'checkbox',
				'defaultvalue' => 1,
				'id' => 'most_views'
			),
			'views_title' => array(
				'label' => $this->T_('Most views title'),
				'type' => 'text',
				'defaultvalue' => 'Most views',
				'size' => 30,
				'id' => 'ministats_views_title'
			),
			'views_limit' => array(
				'label' => $this->T_('Most views limit'),
				'type' => 'integer',
				'defaultvalue' => 5,
				'size' => 3,
				'note' => $this->T_('How many posts should be in the list?'),
				'id' => 'ministats_views_limit'
			),
			'views_more' => array(
				'label' => $this->T_('More link'),
				'type' => 'checkbox',
				'defaultvalue' => 1,
				'id' => 'ministats_views_more'
			),
			'most_words' => array(
				'label' => $this->T_('Most words'),
				'type' => 'checkbox',
				'defaultvalue' => 1,
				'id' => 'most_words'
			),
			'words_title' => array(
				'label' => $this->T_('Most words title'),
				'type' => 'text',
				'defaultvalue' => 'Most words',
				'size' => 30,
				'id' => 'ministats_words_title'
			),
			'words_limit' => array(
				'label' => $this->T_('Most words limit'),
				'type' => 'integer',
				'defaultvalue' => 5,
				'size' => 3,
				'id' => 'ministats_words_limit',
				'note' => $this->T_('How many posts should be in the list?')
			),
			'words_more' => array(
				'label' => $this->T_('More link'),
				'type' => 'checkbox',
				'defaultvalue' => 1,
				'id' => 'ministats_words_more'
			),
		);
	}

	function AdminEndHtmlHead( & $params )
	{
		global $ctrl, $action, $plugins_url;
		if (($ctrl == 'widgets' && $action == 'edit') || ($ctrl == 'plugins' && $action == 'edit_settings'))
		{
			require_js($plugins_url . 'ministats_plugin/ministats.js?v=' . $this->version, false);
		}
		return true;
	}


	/**
	 * Expand the 'aggregate_coll_IDs' setting of a blog for use in SQL
	 *
	 * @return string either " = ID" or " IN (X,Y,..)"
	 */
	function sql_expand_coll_IDs($blog)
	{
		$BlogCache = & get_Cache('BlogCache');
		$Blog = & $BlogCache->get_by_ID($blog);

		$aggregate_coll_IDs = $Blog->get_setting('aggregate_coll_IDs');
		if( empty( $aggregate_coll_IDs ) )
		{	// We only want posts from the current blog:
			return ' = '.$Blog->ID;
		}
		else
		{	// We are aggregating posts from several blogs:
			return ' IN ('.$aggregate_coll_IDs.')';
		}
	}


 	/**
	 * Event handler: SkinTag
	 *
	 * @return boolean did we display?
	 */
	function SkinTag( $params )
	{
	 	global $DB, $Blog;
		/**
		 * Default params:
		 */
		// This is what will enclose the block in the skin:
		if(!isset($params['block_start'])) $params['block_start'] = '<div class="bSideItem">';
		if(!isset($params['block_end'])) $params['block_end'] = "</div>\n";

		// Title:
		if(!isset($params['title'])) $params['title'] = T_('Stats');
		if(!isset($params['block_title_start'])) $params['block_title_start'] = '<h3>';
		if(!isset($params['block_title_end'])) $params['block_title_end'] = '</h3>';


		// This is what will enclose the lists:
		if(!isset($params['before_number'])) $params['before_number'] = '<strong>';
		if(!isset($params['after_number'])) $params['after_number'] = "</strong>";
		if(!isset($params['ministats_subhead_start'])) $params['ministats_subhead_start'] = "<h4>";
		if(!isset($params['ministats_subhead_end'])) $params['ministats_subhead_end'] = "</h4>";

		// This is what will enclose the list:
		if(!isset($params['list_start'])) $params['list_start'] = '<ol>';
		if(!isset($params['list_end'])) $params['list_end'] = "</ol>\n";

		// This is what will separate the list items:
		if(!isset($params['line_start'])) $params['line_start'] = '<li>';
		if(!isset($params['line_end'])) $params['line_end'] = "</li>\n";

		// Before text:
		if(!isset($params['before_text'])) $params['before_text'] = '<p>';
		if(!isset($params['after_text'])) $params['after_text'] = "</p>\n";

		// What comes before and after the counts:
		if(!isset($params['before_count'])) $params['before_count'] = ' <span class="notes">(';
		if(!isset($params['after_count'])) $params['after_count'] = ')</span>';

		// Blog to use.  Default is the current blog being displayed.
		if(!isset($params['blog'])) $params['blog'] = $Blog->ID;

		// The date format:
		if(!isset($params['date_format'])) $params['date_format'] = locale_datefmt();

		if(!isset($params['summary'])) $params['summary'] = true;
		if(!isset($params['summary_text'])) $params['summary_text'] = 'This blog has $post_count$ posts and $comment_count$ comments spanning a range from $first_post_date$ to $last_post_date$.  The total number of words in all posts is $total_words$ and the total number of views for individual posts is $total_views$.';

		if(!isset($params['most_comments'])) $params['most_comments'] = true;
		if(!isset($params['comments_limit'])) $params['comments_limit'] = 5;
		if(!isset($params['comments_title'])) $params['comments_title'] = 'Most comments';

		if(!isset($params['most_views'])) $params['most_views'] = true;
		if(!isset($params['views_title'])) $params['views_title'] = 'Most views';
		if(!isset($params['views_limit'])) $params['views_limit'] = 5;
		if(!isset($params['views_more'])) $params['views_more'] = true;

		if(!isset($params['most_words'])) $params['most_words'] = true;
		if(!isset($params['words_title'])) $params['words_title'] = 'Most words';
		if(!isset($params['words_limit'])) $params['words_limit'] = 5;
		if(!isset($params['words_more'])) $params['words_more'] = true;

		// Start display
		echo $params['block_start'];
		if (!empty($params['title']))
		{
			echo $params['block_title_start'] . $params['title'] . $params['block_title_end'];
		}

		if ($params['summary'])
		{
			echo $params['before_text'];
			// Aggregation from multiple blogs is not currently handled
			$sql = 'SELECT COUNT(DISTINCT post_ID) AS count
                	FROM T_items__item
                	INNER JOIN T_postcats ON post_ID = postcat_post_ID
                	INNER JOIN T_categories ON postcat_cat_ID = cat_ID
                	WHERE cat_blog_ID '.$this->sql_expand_coll_IDs($params['blog']);
			$post_count = number_format($DB->get_var($sql));

			$sql = 'SELECT COUNT(DISTINCT comment_ID)
					FROM T_comments, T_items__item
                	INNER JOIN T_postcats ON post_ID = postcat_post_ID
                	INNER JOIN T_categories ON postcat_cat_ID = cat_ID
					WHERE T_items__item.post_ID = T_comments.comment_post_ID AND
                	cat_blog_ID '.$this->sql_expand_coll_IDs($params['blog']);
			$comment_count = number_format($DB->get_var($sql));

			$sql = 'SELECT post_datestart
					FROM T_items__item
					INNER JOIN T_postcats ON post_ID = postcat_post_ID
					INNER JOIN T_categories ON postcat_cat_ID = cat_ID
					WHERE cat_blog_ID '.$this->sql_expand_coll_IDs($params['blog']). '
					ORDER BY post_datecreated ASC
					LIMIT 1';
			$first_post_date = $DB->get_var($sql);

			$sql = 'SELECT post_datecreated
					FROM T_items__item
					INNER JOIN T_postcats ON post_ID = postcat_post_ID
					INNER JOIN T_categories ON postcat_cat_ID = cat_ID
					WHERE cat_blog_ID '.$this->sql_expand_coll_IDs($params['blog']). '
					ORDER BY post_datecreated DESC
					LIMIT 1';
			$last_post_date = $DB->get_var($sql);

			$sql = 'SELECT SUM(post_wordcount)
					FROM T_items__item
					INNER JOIN T_postcats ON post_ID = postcat_post_ID
					INNER JOIN T_categories ON postcat_cat_ID = cat_ID
					WHERE cat_blog_ID '.$this->sql_expand_coll_IDs($params['blog']);
			$total_words = number_format( $DB->get_var($sql) );

			$sql = 'SELECT SUM(post_views)
					FROM T_items__item
					INNER JOIN T_postcats ON post_ID = postcat_post_ID
					INNER JOIN T_categories ON postcat_cat_ID = cat_ID
					WHERE cat_blog_ID '.$this->sql_expand_coll_IDs($params['blog']);
			$total_views = number_format( $DB->get_var($sql) );


			$search = array(
				'$post_count$',
				'$comment_count$',
				'$first_post_date$',
				'$last_post_date$',
				'$total_words$',
				'$total_views$'
				);
			$replace = array(
				$params['before_number'] . $post_count . $params['after_number'],
				$params['before_number'] . $comment_count . $params['after_number'],
				$params['before_number'].mysql2date( $params['date_format'], $first_post_date).$params['after_number'],
				$params['before_number'].mysql2date( $params['date_format'], $last_post_date).$params['after_number'],
				$params['before_number'] . $total_words . $params['after_number'],
				$params['before_number'] . $total_views . $params['after_number'],
				);

			$summary_text = str_replace($search, $replace, $params['summary_text']);

			echo $summary_text;
		}

		if ($params['most_comments'])
		{
			// Posts with the most comments
	        if (!empty($params['comments_title']))
			{
				echo $params['ministats_subhead_start'] . $params['comments_title'] . $params['ministats_subhead_end'];
			}

			$sql = "SELECT DISTINCT post_ID , post_title, post_urltitle ,
			ROUND(COUNT(post_ID) / COUNT(DISTINCT postcat_cat_ID)) AS count
			FROM T_comments , T_items__item
			INNER JOIN T_postcats ON post_ID = postcat_post_ID
			INNER JOIN T_categories ON postcat_cat_ID = cat_ID
			WHERE T_items__item.post_ID = T_comments.comment_post_ID
			AND cat_blog_ID ".$this->sql_expand_coll_IDs($params['blog']);
			$sql .= " group by post_ID  ORDER BY `count`  DESC
			LIMIT ".$params['comments_limit'];

			$this->ministats_list($sql, $params);
		}

		if ($params['most_views'])
		{
			if (!empty($params['views_title']))
			{
				echo $params['ministats_subhead_start'] . $params['views_title'] . $params['ministats_subhead_end'];
			}

			$list = new ItemList2($Blog);
			$list->set_filters(array(
				'orderby' => 'views',
				'order' => 'desc',
				'unit' => 'posts',
				'posts' => $params['views_limit']
			));
			$list->query();
			echo $params['list_start'];
	      	while ($views_item = $list->get_item()) {
	        	echo $params['line_start'];
				$views_item->title();
	 			echo $params['before_count'];
				echo number_format( $views_item->views );
				echo $params['after_count'];
	            echo $params['line_end'];
  		  	}
            if ( $params['views_more'] ) {
				echo '<a href="';
				echo url_add_param( $Blog->gen_blogurl( 'default' ), 'orderby=views' );
				echo '" title="Reorder blog by views">More...</a>';
			}
			echo $params['list_end'];
		}

		if ($params['most_words'])
		{
		    if (!empty($params['words_title']))
			{
				echo $params['ministats_subhead_start'] . $params['words_title'] . $params['ministats_subhead_end'];
			}

			$list = new ItemList2($Blog);
			$list->set_filters(array(
				'orderby' => 'wordcount',
				'order' => 'desc',
				'unit' => 'posts',
				'posts' => $params['words_limit']
			));
			$list->query();
			echo $params['list_start'];
	      	while ($views_item = $list->get_item()) {
	        	echo $params['line_start'];
				$views_item->title();
	 			echo $params['before_count'];
				echo number_format( $views_item->wordcount );
				echo $params['after_count'];
	            echo $params['line_end'];
  		  	}
            if ( $params['words_more'] ) {
				echo '<a href="';
				echo url_add_param( $Blog->gen_blogurl( 'default' ), 'orderby=wordcount' );
				echo '" title="Reorder blog by wordcount">More...</a>';
			}
			echo $params['list_end'];
		}

		echo $params['block_end'];
		return true;
	}

	function ministats_list($sql, $params)
	{
		global $Blog, $DB;

		$results = $DB->get_results($sql);

		echo $params['list_start'];
		foreach ($results as $result)
		{
			//$ids[] = $result->post_ID;
		//}

			$list = new ItemList2($Blog);
			//pre_dump($list->ItemQuery);
			//$list->ItemQuery->select = 'DISTINCT post_ID , post_title, post_urltitle ,
			//ROUND(COUNT(post_ID) / COUNT(DISTINCT postcat_cat_ID)) AS count';
			$list->ItemQuery->where = "post_id = '" . $result->post_ID . "'" ;
			$list->query();
			//pre_dump($list->ItemQuery);
			//pre_dump($list->ItemQuery);


			while ($Item = $list->get_item())
			{
				//pre_dump($Item->title);
				echo $params['line_start'];
				$Item->title();
				//pre_dump($result->count);
				echo $params['before_count'];
				echo number_format($result->count);
				echo $params['after_count'];
				echo $params['line_end'];
			}
		}
		echo $params['list_end'];

	}
}

?>
